@@ -1,5 +1,8 @@
 package Perinci::Sub::Wrapper;
 
+our $DATE = '2015-01-11'; # DATE
+our $VERSION = '0.68'; # VERSION
+
 use 5.010001;
 use strict;
 use warnings;
@@ -15,11 +18,13 @@ our @EXPORT_OK = qw(wrap_sub);
 
 our $Log_Wrapper_Code = $ENV{LOG_PERINCI_WRAPPER_CODE} // 0;
 
-our $VERSION = '0.67'; # VERSION
-our $DATE = '2014-12-10'; # DATE
-
 our %SPEC;
 
+$SPEC{':package'} = {
+    v => 1.1,
+    summary => 'A multi-purpose subroutine wrapping framework',
+};
+
 # "protocol version" (v). whenever there's a significant change in the basic
 # structure of the wrapper, which potentially cause some/a lot of property
 # handlers to stop working, we increase this. property handler must always state
@@ -663,7 +668,7 @@ sub handle_result {
     my $opt_sin = $self->{_args}{_schema_is_normalized};
     my $opt_vr  = $self->{_args}{validate_result};
 
-    my %ss; # key = status, value = schema
+    my %schemas_by_status; # key = status, value = schema
 
     # collect and check handlers
     my %handler_args;
@@ -676,7 +681,7 @@ sub handle_result {
         # check builtin result spec key
         next if $k =~ /\A(
                            summary|description|tags|default_lang|
-                           schema|statuses|
+                           schema|statuses|stream|
                            x
                        )\z/x;
         # try a property module first
@@ -713,25 +718,38 @@ sub handle_result {
     # validate result
     my @modules;
     if ($v->{schema} && $opt_vr) {
-        $ss{200} = $v->{schema};
+        $schemas_by_status{200} = $v->{schema};
     }
     if ($v->{statuses} && $opt_vr) {
         for my $s (keys %{$v->{statuses}}) {
             my $sv = $v->{statuses}{$s};
             if ($sv->{schema}) {
-                $ss{$s} = $sv->{schema};
+                $schemas_by_status{$s} = $sv->{schema};
             }
         }
     }
 
     my $sub_name = $self->{_args}{sub_name};
-    if (keys %ss) {
+
+    if ($v->{stream}) {
+
+        $self->select_section('after_call_res_validation');
+        $self->push_lines("my \$_w_res2 = \$_w_res->[2];");
+        $self->_add_module('Scalar::Util');
+        $self->_errif(
+            500,
+            q["Stream result must be either a glob, object that supports getitem/getline, or a (tied) array"],
+            '!(ref($_w_res2) eq "GLOB" || (Scalar::Util::blessed($_w_res2) && ($_w_res2->can("getitem") || $_w_res2->can("getline")) || ref($_w_res2) eq "ARRAY"))',
+        );
+
+    } elsif (keys %schemas_by_status) {
+
         $self->select_section('after_call_res_validation');
         $self->push_lines("my \$_w_res2 = \$_w_res->[2];");
         $self->push_lines("my \$_w_err2_res;");
 
-        for my $s (sort keys %ss) {
-            my $sch = $ss{$s};
+        for my $s (sort keys %schemas_by_status) {
+            my $sch = $schemas_by_status{$s};
             my $cd = $self->_plc->compile(
                 data_name            => '_w_res2',
                 # err_res can clash on arg named 'res'
@@ -755,7 +773,8 @@ sub handle_result {
                 "\$_w_err2_res");
             $self->unindent;
             $self->push_lines("}");
-        }
+        } # for schemas_by_status
+
     }
 }
 
@@ -1246,7 +1265,7 @@ Perinci::Sub::Wrapper - A multi-purpose subroutine wrapping framework
 
 =head1 VERSION
 
-This document describes version 0.67 of Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2014-12-10.
+This document describes version 0.68 of Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2015-01-11.
 
 =head1 SYNOPSIS
 
@@ -1333,7 +1352,7 @@ wrapping code to source code during build.
 
 =head2 wrap_sub(%args) -> [status, msg, result, meta]
 
-Wrap subroutine to do various things, like enforcing Rinci properties.
+{en_US Wrap subroutine to do various things, like enforcing Rinci properties}.
 
 Arguments ('*' denotes required arguments):
 
@@ -1341,76 +1360,90 @@ Arguments ('*' denotes required arguments):
 
 =item * B<compile> => I<bool> (default: 1)
 
-Whether to compile the generated wrapper.
+{en_US Whether to compile the generated wrapper}.
 
+{en_US 
 Can be set to 0 to not actually wrap but just return the generated wrapper
 source code.
+}
 
 =item * B<convert> => I<hash>
 
-Properties to convert to new value.
+{en_US Properties to convert to new value}.
 
+{en_US 
 Not all properties can be converted, but these are a partial list of those that
 can: v (usually do not need to be specified when converting from 1.0 to 1.1,
 will be done automatically), args_as, result_naked, default_lang.
+}
 
 =item * B<debug> => I<bool> (default: 0)
 
-Generate code with debugging.
+{en_US Generate code with debugging}.
 
+{en_US 
 If turned on, will produce various debugging in the generated code. Currently
 what this does:
 
 =over
 
 =item * add more comments (e.g. for each property handler)
+}
 
 =back
 
 =item * B<meta>* => I<hash>
 
-The function metadata.
+{en_US The function metadata}.
 
 =item * B<meta_name> => I<str>
 
-Where to find the metadata, e.g. "$SPEC{foo}".
+{en_US Where to find the metadata, e.g. "$SPEC{foo}"}.
 
+{en_US 
 Some wrapper code (e.g. handler for C<dep> property) needs to refer to the
 function metadata. If not provided, the wrapper will store the function metadata
 in a unique variable (e.g. C<$Perinci::Sub::Wrapped::meta34127816>).
+}
 
 =item * B<sub> => I<str>
 
-The code to be wrapped.
+{en_US The code to be wrapped}.
 
+{en_US 
 At least one of C<sub> or C<sub_name> must be specified.
+}
 
 =item * B<sub_name> => I<str>
 
-The name of the subroutine, e.g. func or Foo::func (qualified).
+{en_US The name of the subroutine, e.g. func or Foo::func (qualified)}.
 
+{en_US 
 At least one of C<sub> or C<sub_name> must be specified.
+}
 
 =item * B<validate_args> => I<bool>
 
-Whether wrapper should validate arguments.
+{en_US Whether wrapper should validate arguments}.
 
+{en_US 
 If set to true, will validate arguments. Validation error will cause status 400
 to be returned. The default is to enable this unless previous wrapper(s) have
 already done this.
+}
 
 =item * B<validate_result> => I<bool>
 
-Whether wrapper should validate arguments.
+{en_US Whether wrapper should validate arguments}.
 
+{en_US 
 If set to true, will validate sub's result. Validation error will cause wrapper
 to return status 500 instead of sub's result. The default is to enable this
 unless previous wrapper(s) have already done this.
+}
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -1420,8 +1453,10 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
-The wrapped subroutine along with its new metadata (hash)
+Return value: {en_US The wrapped subroutine along with its new metadata} (hash)
 
+
+{en_US 
 Aside from wrapping the subroutine, the wrapper will also create a new metadata
 for the subroutine. The new metadata is a clone of the original, with some
 properties changed, e.g. schema in C<args> and C<result> normalized, some values
@@ -1431,6 +1466,7 @@ The new metadata will also contain (or append) the wrapping log located in the
 C<x.perinci.sub.wrapper.logs> attribute. The wrapping log marks that the wrapper
 has added some functionality (like validating arguments or result) so that
 future nested wrapper can choose to avoid duplicating the same functionality.
+}
 
 =for Pod::Coverage ^(new|handle(meta)?_.+|wrap|add_.+|section_empty|indent|unindent|get_indent_level|select_section|push_lines)$
 
@@ -1603,7 +1639,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Su
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Perinci-Sub-Wrapper>.
+Source repository is at L<https://github.com/sharyanto/perl-Perinci-Sub-Wrapper>.
 
 =head1 BUGS
 
@@ -1619,7 +1655,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by perlancar@cpan.org.
+This software is copyright (c) 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
